using ProjectTank.CollectionGenericObjects;
using ProjectTank.Drawning;

namespace ProjectTank
{
    /// <summary>
    /// Форма работы с компанией и её коллекцией
    /// </summary>
    public partial class FormBattleTankCollection : Form
    {
        /// <summary>
        /// Хранилище коллекций
        /// </summary>
        private readonly StorageCollection<DrawningTank> _storageCollection;
        /// <summary>
        /// Компания
        /// </summary>
        private AbstractCompany? _company = null;

        /// <summary>
        /// Конструктор
        /// </summary>
        public FormBattleTankCollection()
        {
            InitializeComponent();
            _storageCollection = new();
        }

        /// <summary>
        /// Выбор компании
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ComboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
        {
            panelCompanyTools.Enabled = false;
        }

        /// <summary>
        /// Создание объекта класса-перемещения
        /// </summary>
        /// <param name="type">Тип создоваемого объекта</param>
        private void SetTank(DrawningTank tank)
        {
            if (_company == null || tank == null) 
                return;

            if (_company + tank != -1)
            {
                MessageBox.Show("Объект добавлен");
                pictureBox.Image = _company.Show();
            }
            else
            {
                MessageBox.Show("Не удалось добавить объект");
            }
        }

        /// <summary>
        /// добавление танка
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddTank_Click(object sender, EventArgs e)
        {
            if (_company == null)
            {
                return;
            }
            FormTankConfig form = new();
            form.AddEvent(SetTank);
            form.Show();
        }
        /// <summary>
        /// Удаление танка
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonRemoveTank_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(maskedTextBox.Text) || _company == null) return;

            if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No) return;

            int position = Convert.ToInt32(maskedTextBox.Text);

            if (_company - position != null)
            {
                MessageBox.Show("Объект удален");
                pictureBox.Image = _company.Show();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }
        /// <summary>
        /// передать на тесты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonGoToCheck_Click(object sender, EventArgs e)
        {
            if (_company == null) return;

            DrawningTank? tank = null;
            int coutner = 100;

            while (tank == null)
            {
                tank = _company.GetRandomObject();
                coutner--;
                if (coutner <= 0) break;
            }

            if (tank == null) return;

            FormBattleTank form = new()
            {
                SetTank = tank
            };
            form.ShowDialog();
        }
        /// <summary>
        /// обновление
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRefresh_Click(object sender, EventArgs e)
        {
            if (_company == null) return;
            pictureBox.Image = _company.Show();
        }
        /// <summary>
        ///  Добавление коллекции
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCollectionAdd_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMassive.Checked))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            CollectionType collectionType = CollectionType.None;
            if (radioButtonMassive.Checked)
            {
                collectionType = CollectionType.Massive;
            }
            else if (radioButtonList.Checked)
            {
                collectionType = CollectionType.List;
            }

            _storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
            RerfreshListBoxItems();
        }

        /// <summary>
        /// Удвление коллекции
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCollectionDel_Click(object sender, EventArgs e)
        {
            if (listBoxСollection.SelectedIndex < 0 || listBoxСollection.SelectedItem == null)
            {
                MessageBox.Show("Коллекция не выбрана");
                return;
            }
            if (MessageBox.Show("Удалить коллекцию?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            _storageCollection.DelCollection(listBoxСollection.SelectedItem.ToString());
            RerfreshListBoxItems();
        }
        /// <summary>
        /// Обновление списка в listBoxCollection
        /// </summary>
        private void RerfreshListBoxItems()
        {
            listBoxСollection.Items.Clear();
            for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
            {
                string? colName = _storageCollection.Keys?[i];
                if (!string.IsNullOrEmpty(colName))
                {
                    listBoxСollection.Items.Add(colName);
                }
            }

        }
        /// <summary>
        ///  Создание компании
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCreateCompany_Click(object sender, EventArgs e)
        {
            if (listBoxСollection.SelectedIndex < 0 || listBoxСollection.SelectedItem == null)
            {
                MessageBox.Show("Коллекция не выбрана");
                return;
            }

            ICollectionGenericObjects<DrawningTank>? collection = _storageCollection[listBoxСollection.SelectedItem.ToString() ?? string.Empty];
            if (collection == null)
            {
                MessageBox.Show("Коллекция не проинициализирована");
                return;
            }

            switch (comboBoxSelectorCompany.Text)
            {
                case "Хранилище":
                    _company = new TankBase(pictureBox.Width, pictureBox.Height, collection);
                    break;
            }

            panelCompanyTools.Enabled = true;
            RerfreshListBoxItems();
        }
    }
}