﻿using ProjectTank.Drawning;

namespace ProjectTank.MovementStrategy
{
    public class MoveableTank : IMoveableObject
    {
        /// <summary>
        /// Поле-объект класса  или его DrawningTank наследника
        /// </summary>
        private readonly DrawningTank? _tank = null;
        /// <summary>
        /// конструктор
        /// </summary>
        /// <param name="tank">Объект класса DrawningTank</param>
        public MoveableTank(DrawningTank tank)
        {
            _tank = tank;
        }
        public ObjectParameters? GetObjectPosition
        {
            get
            {
                if (_tank == null || _tank.EntityTank == null ||
                !_tank.GetPosX.HasValue || !_tank.GetPosY.HasValue)
                {
                    return null;
                }
                return new ObjectParameters(_tank.GetPosX.Value,
                _tank.GetPosY.Value, _tank.GetWidth, _tank.GetHeight);
            }
        }
        public int GetStep => (int)(_tank?.EntityTank?.Step ?? 0);
        public bool TryMoveObject(MovementDirection direction)
        {
            if (_tank == null || _tank.EntityTank == null)
            {
                return false;
            }
            return _tank.MoveTransport(GetDirectionType(direction));
        }
        /// <summary>
        /// Конвертация из MovementDirection в DirectionType
        /// </summary>
        /// <param name="direction">MovementDirection</param>
        /// <re turns>DirectionType</returns>
        private static DirectionType GetDirectionType(MovementDirection direction)
        {
            return direction switch
            {
                MovementDirection.Left => DirectionType.Left,
                MovementDirection.Right => DirectionType.Right,
                MovementDirection.Up => DirectionType.Up,
                MovementDirection.Down => DirectionType.Down,
                _ => DirectionType.Unknow,
            };
        }
    }
}
